package Cyrus::SIEVE::Manage;
#
# $Id: Manage.pm 3 2008-06-04 11:38:55Z martin $
#
# Copyright (c) 2005 Martin Werthmoeller <mw@werthmoeller.de>
#

use Cyrus::SIEVE::managesieve;
use Carp;
use strict;

$Cyrus::SIEVE::Manage::VERSION = '0.2';
$Cyrus::SIEVE::Manage::activatedScript = '';
@Cyrus::SIEVE::Manage::ScriptList      = ();

sub new {
    my $this = shift;
    my $user = shift || croak "No username given.\n";
    my $pass = shift || croak "No password given.\n";
    my $host = shift || 'localhost';
    my $class = ref($this) || $this;
    my $self = { 'username' => $user,
                 'password' => $pass,
                 'hostname' => $host,
                 'authname' => '',
                 'ERROR'    => '',
               };
    bless $self, $class;

    return $self;
}


# callbacks
sub Cyrus::SIEVE::Manage::username { return $Cyrus::SIEVE::Manage::username; };
sub Cyrus::SIEVE::Manage::authname { return $Cyrus::SIEVE::Manage::authname; };
sub Cyrus::SIEVE::Manage::password { return $Cyrus::SIEVE::Manage::password; };
sub Cyrus::SIEVE::Manage::realm    { return $Cyrus::SIEVE::Manage::realm; };
sub connect
{
    my $self = shift;
    $self->{'authname'} = shift || $self->{'username'};
    $Cyrus::SIEVE::Manage->{$self}->{'authname'} = $self->{'authname'};

    $Cyrus::SIEVE::Manage::authname = $self->{'authname'};
    $Cyrus::SIEVE::Manage::username = $self->{'username'};
    $Cyrus::SIEVE::Manage::password = $self->{'password'};
    $Cyrus::SIEVE::Manage::realm    = $self->{'realm'};
    $self->{'handle'} = sieve_get_handle($self->{'hostname'},
                                         'Cyrus::SIEVE::Manage::authname',
                                         'Cyrus::SIEVE::Manage::username',
                                         'Cyrus::SIEVE::Manage::password',
                                         'Cyrus::SIEVE::Manage::realm',
                                        );
    if ($self->{'handle'}) {
        return $self;
    } else {
        $self->_setError("Unable to connect to server ".$self->{'hostname'});
        croak "Unable to connect to the server\n";
    }
    return $self;
}


sub error
{
    my $self = shift;
    my $msg = $self->{'ERROR'};
    $self->{'ERROR'} = '';
    return $msg;
}


sub put
{
    my ($self,$file,$upstreamname) = @_;
    my $err;

    if ($upstreamname) {
        $err = sieve_put_file_withdest($self->{'handle'}, $file, $upstreamname);
    }
    else {
        $err = sieve_put_file($self->{'handle'}, $file);
    }
    if ($err) {
        $self->_setError();
        return 1;
    }
    return 0;
}


sub activate
{
    my ($self,$filename) = @_;
    my $err = sieve_activate($self->{'handle'},$filename);
    if ($err) {
        $self->_setError();
        croak "Cannot activate script: $err\n";
    }
    return $self;
}


sub delete
{
    my ($self,$filename) = @_;
    my $err = sieve_delete($self->{'handle'},$filename);
    if ($err) {
        $self->_setError();
        croak "Cannot delete script: $err\n";
    }
    return $self;
}


sub get
{
    my ($self,$filename) = @_;
    my $script;
    my $err = sieve_get($self->{'handle'},$filename,$script);
    if ($err) {
        $self->_setError();
        croak "Cannot get script: $err\n";
    }
    return $script;
}


sub list
{
    my ($self) = @_;
    @Cyrus::SIEVE::Manage::ScriptList = ();
    my $err = sieve_list($self->{'handle'},'Cyrus::SIEVE::Manage::_list_callback');
    if ($err) {
        $self->_setError();
        croak "Cannot list scripts: $err\n";
    }
    return @Cyrus::SIEVE::Manage::ScriptList;
}


sub getActive
{
    my ($self) = @_;
    my $err = sieve_list($self->{'handle'},'Cyrus::SIEVE::Manage::_getActive_callback');
    if ($err) {
        $self->_setError();
        croak "Cannot activate script: $err\n";
    }
    return $Cyrus::SIEVE::Manage::activatedScript;
}


########################################################################
# private methods
########################################################################


sub Cyrus::SIEVE::Manage::_list_callback {
  my($file,$active) = @_ ;
  push @Cyrus::SIEVE::Manage::ScriptList, $file;
}

sub Cyrus::SIEVE::Manage::_getActive_callback {
  my($file,$active) = @_ ;
  if ($active) { $Cyrus::SIEVE::Manage::activatedScript = $file; }
}


# sub _setError([$str])
#
# Sets the error string. If $str is given, ERROR
# will be set to this value, otherwise the error string is set by
# calling sieve_get_error().
# Returns the current object.
sub _setError
{
    my ($self,$str) = @_;
    if ($str) {
        $self->{'ERROR'} = $str,
    }
    else {
        $self->{'ERROR'} = sieve_get_error($self->{'handle'});
    }
    return $self;
}


### EOP ###
1;
__END__


=head1 NAME

Cyrus::SIEVE::Manage - Interface to the Cyrus sieve functions

=head1 SYNOPSIS

    use C<Cyrus::SIEVE::Manage>;

    $Sieve = new Cyrus::SIEVE::Manage('cyrus','secret');
    $Sieve->connect('martin');

    my $err = $Sieve->put('/home/martin/scripts/spamfilter.sieve','spamfilter');
    if ($err) {
        print "An error occured: ",$Sieve->error,"\n";
        exit 1;
    }
    $Sieve->activate('spamfilter');


=head1 DESCRIPTION

The Cyrus Sieve implementation ships with the Perl module
Cyrus::SIEVE::managesieve to handle sieve scripts for the Cyrus IMAP
server. The interface of the Cyrus::SIEVE::managesieve module is a
little bit strange and poor documented.

This Cyrus::SIEVE::Manage module provides an object oriented interface to the
functions of Cyrus::SIEVE::managesieve

The module will throw (croak) an exception on errors, thus you should handle
method calls with eval.

=head1 METHODS

=head2 new(<$authname>,<$password>[, $hostname])

Constructor for an Cyrus::SIEVE::Manage object. The parameters are:

  * authname - Username of the user to connect to the server.

  * password - Password of the user to connect.

  * hostname - Hostname of the cyrus sieve server to
    connect. Optional, default is localhost.


=head2 connect([$user])

Connect to the sieve server as user $authname given at the
constructor. If the optional parameter $user is given, the mailbox of
$user will be connected if $authname is authorized to edit the mailbox
of $user.

Returns the current object if successul or a false value if failed.


=head2 error()

Returns the last errorstring and clears the ERROR attribute.


=head2 put(<$filename>[, $upstreamname])

Loads the file $filename to the server. If $upstreamname is given, the
file will be stored as $upstreamname.

Returns the current object if successful, otherwise a false value.


=head2 activate([$filename])

Activates the upstream sieve script $filename. If no filename is
given, the current script will be deactivated.

Returns the current object if successful, otherwise a false value.


=head2 delete($filename)

Deletes the upstream sieve script $filename.
Returns the current object if successful, otherwise a false value.


=head2 get()

Fetches the upstream sieve script $filename.
Returns the script as a scalar if successful, otherwise a false value.


=head2 list()

Fetches a list of upstream sieve scripts
Returns an array with the list if successful, otherwise a false value.


=head2 getActive()

Returns the name of the currently activated script.


=head1 BUGS

Bugs? Features!

=head1 SEE ALSO

perl(1), Cyrus::SIEVE::managesieve

=head1 AUTHOR

 Copyright (c) 2005 Martin Werthmoeller <mw@werthmoeller.de>

 The code of this module is derivated from the sieveshell program of
 cyrus.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Fre e Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILIT Y or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 5 9 Temple Place, Suite 330, Boston, MA
   02111-1307 USA

=cut
